@@ -6,6 +6,7 @@ This module is ued for estimating and selecting models
 
 import numpy as np
 from matplotlib import pyplot as plt
+from maysics import data_split
 
 
 class Estimate():
@@ -14,16 +15,19 @@ class Estimate():
     ----
     data：ndarray形式，自变量
     targets：ndarray形式，因变量
+    verbose：整型，可选，可以是0、1、2，默认为0
     
     
     Parameters
     ----------
     data: ndarray
     targets: ndarray
+    verbose: int, callable, can be 0, 1, 2, default = 0
     '''
-    def __init__(self, data, targets):
+    def __init__(self, data, targets, verbose=0):
         self.data = data
         self.targets = targets
+        self.__verbose = verbose
     
     
     def __part_of_kfold(self, num_validation_samples, num_part):
@@ -38,9 +42,78 @@ class Estimate():
                                         self.targets[num_validation_samples * (num_part + 1):]])
         
         return train_data, train_targets, validation_data, validation_targets
+
+
+    def keras_normal(self, build_model, num_epochs, batch_size, train_size=None, val_size=None, param=None, shuffle=True, random_state=None):
+        ''' 
+        对keras模型或与keras模型相同架构的模型进行数据分割和训练
+        
+        参数
+        ----
+        build_model：模型函数，需要返回一个model
+        num_epochs：整型，迭代次数
+        batch_size：整型，批量大小
+        train_size：浮点数类型，可选，训练集占总数据量的比，取值范围为(0, 1]，默认为0.75
+        val_size：浮点数类型，可选，测试集占总数据量的比，取值范围为[0, 1)，当train_size被定义时，该参数无效
+        shuffle：布尔类型，可选，True表示打乱数据，False表示不打乱数据，默认为True
+        param：字典类型，可选，当build_model函数有参数时，需输入以参数名为键，参数值为值的字典
+        random_state：整型，可选，随机种子
+        
+        
+        Spliting data and training for keras models and other similar models
+        
+        Parameters
+        ----------
+        build_model: the function to generate keras-model, which should return a model
+        num_epochs: int, the number of epochs
+        batch_size: int, the size of batch
+        train_size: float, callable, ratio of training set to total data, value range is (0, 1], default=0.75
+        val_size: float, callable, ratio of test set to total data, value range is [0, 1)
+        shuffle: bool, callable, 'True' will shuffle the data, 'False' will not, default = True
+        param: dict, callable, when build_model() has non-default parameters, param needs to be input a dictionary with parm_name as key and param_value as value
+        random_state: int, callable, random seed
+        '''
+        self.__num_epochs = num_epochs
+        val_loss = np.zeros(num_epochs)
+        val_acc = np.zeros(num_epochs)
+        loss = np.zeros(num_epochs)
+        acc = np.zeros(num_epochs)
+        np.random.seed(random_state)
+        
+        if shuffle:
+            local_data = self.data
+            local_targets = self.targets
+            state = np.random.get_state()
+            np.random.shuffle(local_data)
+            np.random.set_state(state)
+            np.random.shuffle(local_targets)
+        
+        if not (train_size or val_size):
+            train_size = 0.75
+        elif val_size:
+            train_size = 1 - val_size
+    
+        if train_size <= 0 or train_size > 1:
+            raise Exception("'train_size' should be in (0, 1], 'test_size' should be in [0, 1)")
+        
+        num_of_data = int(len(local_data) * train_size)
+        train_data = local_data[:num_of_data]
+        train_targets = local_targets[:num_of_data]
+        validation_data = local_data[num_of_data:]
+        validation_targets = local_targets[num_of_data * train_size:]
+                
+        if param:
+            model = build_model(**param)
+        else:
+            model = build_model()
+        
+        history = model.fit(train_data, train_targets,
+                            validation_data=(validation_data, validation_targets),
+                            epochs=num_epochs, batch_size=batch_size, verbose=self.__verbose)
+        self.history = history.history
     
     
-    def keras_rekfold(self, build_model, num_epochs, batch_size, k=5, n=3, verbose=0, shuffle=True, param=None, random_state=None):
+    def keras_rekfold(self, build_model, num_epochs, batch_size, k=5, n=3, shuffle=True, param=None, random_state=None):
         '''
         对keras模型或与keras模型相同架构的模型进行重复的k折验证
         
@@ -51,13 +124,12 @@ class Estimate():
         batch_size：整型，批量大小
         k：整型，可选，k折验证的折叠数，默认为5
         n：整型，可选，k折验证的重复次数，默认为3
-        verbose：整型，可选，可以是0、1、2，默认为0
         shuffle：布尔类型，可选，True表示打乱数据，False表示不打乱数据，默认为True
         param：字典类型，可选，当build_model函数有参数时，需输入以参数名为键，参数值为值的字典
         random_state：整型，可选，随机种子
         
         
-        Repeated k-fold verification for the keras model and other similar models
+        Repeated k-fold verification for keras models and other similar models
         
         Parameters
         ----------
@@ -66,7 +138,6 @@ class Estimate():
         batch_size: int, the size of batch
         k: int, callable, the number of k-fold, default = 5
         n: int, callable, the number of repetition of k-fold, default = 3
-        verbose: int, callable, can be 0, 1, 2, default = 0
         shuffle: bool, callable, 'True' will shuffle the data, 'False' will not, default = True
         param: dict, callable, when build_model() has non-default parameters, param needs to be input a dictionary with parm_name as key and param_value as value
         random_state: int, callable, random seed
@@ -98,7 +169,7 @@ class Estimate():
                     model = build_model()
                 history = model.fit(train_data, train_targets,
                                     validation_data=(validation_data, validation_targets),
-                                    epochs=num_epochs, batch_size=batch_size, verbose=verbose)
+                                    epochs=num_epochs, batch_size=batch_size, verbose=self.__verbose)
                 if i == 0 and j == 0:
                     new_keys = list(history.history.keys())
                     new_values_of_history = np.zeros_like(list(history.history.values()))
@@ -110,7 +181,6 @@ class Estimate():
         self.history = history_dict
     
     
-    
     def __img_process(self, epochs_range, metrics_list):
         epochs_range_list = np.arange(epochs_range[0], epochs_range[1]+1)
         
@@ -195,6 +265,7 @@ class Search():
     ----
     data：自变量数据
     targets：目标数据
+    verbos：整型，可选，可以是0、1、2，默认为0
     
     属性
     ----
@@ -207,14 +278,16 @@ class Search():
     ----------
     data: independent variable
     targets: dependent variable
+    verbose: int, callable, can be 0, 1, 2, default = 0
     
     Atrributes
     ----------
     comb: the best combination
     '''
-    def __init__(self, data, targets):
+    def __init__(self, data, targets, verbose=0):
         self.data = data
         self.targets = targets
+        self.__verbose = verbose
     
     
     def __part_of_kfold(self, num_validation_samples, num_part):
@@ -302,7 +375,7 @@ class Search():
                                 'epoch': final_epoch}
     
     
-    def gridsearch(self, build_model, num_epochs, batch_size, param, k=5, n=1, select_method=('loss', 'min'), verbose=0, shuffle=True, random_state=None):
+    def gridsearch(self, build_model, num_epochs, batch_size, param, k=5, n=1, select_method=('loss', 'min'), shuffle=True, random_state=None):
         '''
         参数
         ----
@@ -313,7 +386,6 @@ class Search():
         k：整型，可选，k折验证的折叠数，默认为5
         n：整型，可选，k折验证的重复次数，默认为1
         select_method：元组类型，评估方法，形式为('指标', 'max'或'min')，默认为('loss', 'min')
-        verbos：整型，可选，可以是0、1、2，默认为0
         shuffle：布尔类型，可选，True表示打乱数据，False表示不打乱数据，默认为True
         random_state：整型，可选，随机种子
         
@@ -327,7 +399,6 @@ class Search():
         k: int, callable, the number of k-fold, default = 5
         n: int, callable, the number of repetition of k-fold, default = 1
         select_method: tuple, method of selecting the best collocation, default = ('loss', 'min')
-        verbose: int, callable, can be 0, 1, 2, default = 0
         shuffle: bool, callable, 'True' will shuffle the data, 'False' will not, default = True
         random_state: int, callable, random seed
         '''
@@ -337,7 +408,7 @@ class Search():
             Search.__part_of_gridsearch(self, build_model=build_model, num_epochs=num_epochs,\
                                         batch_size=batch_size, param=param,\
                                         k=k, n=n, select_method=select_method,\
-                                        verbose=verbose, shuffle=shuffle)
+                                        verbose=self.__verbose, shuffle=shuffle)
         else:
             last_dict_list = []
             last_val_m_list = []
@@ -348,7 +419,7 @@ class Search():
                 Search.__part_of_gridsearch(self, build_model=build_model, num_epochs=num_epochs,\
                                             batch_size=batch_size, param=param,\
                                             k=k, n=n, select_method=select_method,\
-                                            verbose=verbose, shuffle=shuffle)
+                                            verbose=self.__verbose, shuffle=shuffle)
                 last_dict_list.append(last_dict)
                 last_val_m_list.append(last_val_m)
                 last_epoch_list.append(last_epoch)
@@ -375,14 +446,18 @@ class Error():
     
     参数
     ----
-    model：一个类，该类必须要有predict函数
+    func：函数类型，模型的预测函数
     
     属性
     ----
-    error_mat：一维ndarray数组，误差绝对值矩阵
-    error_list：误差绝对值从大到小的排序
+    abs_error：1-D ndarray数组，绝对误差列表
+    rel_error：1-D ndarray数组，相对误差列表
+    abs_sort：绝对误差从大到小的排序
+    rel_sort：相对误差从大到小的排序
     mae：平均绝对误差
+    mape：平均绝对百分比误差
     mse：平均平方误差
+    rmse：均方根误差
     sae：绝对误差和
     sse：残差平方和
     
@@ -391,37 +466,67 @@ class Error():
     
     Parameter
     ---------
-    model: class, this class should have 'predict' function
+    func: function, predicting function of models
     
     Atrributes
     ----------
-    error_mat: one-dimension ndarray, absolute error matrix
-    error_list: list of absolute values of errors sorted from large to small
+    abs_error: 1-D ndarray, absolute error list
+    rel_error: 1-D ndarray, relative error list
+    abs_sort: list of absolute values of errors sorted from large to small
+    rel_sort: list of relative values of errors sorted from large to small
     mae: mean absolute error
+    mape: mean absolute percentage error
     mse: mean squared error
+    rmse: root mean square error
     sae: sum of absolute error
     sse: sum of squared error
     '''
-    def __init__(self, model):
-        self.__model = model
+    def __init__(self, func):
+        self.__func = func
     
     
     def fit(self, data, target):
         data = np.array(data)
         target = np.array(target)
-        num_data = len(target)
+        self.__num_data = len(target)
         
-        predict_target = self.__model.predict(data)
-        self.error_mat = abs(target - predict_target)
-        
-        mat_index = np.arange(num_data)
-        self.error_list = sorted(list(zip(self.error_mat, mat_index)), reverse=True)
+        predict_target = self.__func(data)
+        self.abs_error = abs(target - predict_target)
+        self.rel_error = abs((target - predict_target) / target)
     
-        self.sse = sum(self.error_mat**2)
-        self.sae = sum(self.error_mat)
-        
-        self.mse = self.sse / num_data
-        self.mae = self.sae / num_data
+    @property
+    def abs_sort(self):
+        error_index = np.arange(self.__num_data)
+        self.abs_sort = sorted(list(zip(self.abs_error, error_index)), reverse=True)
+    
+    @property
+    def rel_sort(self):
+        error_index = np.arange(self.__num_data)
+        self.rel_sort = sorted(list(zip(self.rel_error, error_index)), reverse=True)
+    
+    @property
+    def sse(self):
+        return sum(self.abs_error**2)
+    
+    @property
+    def sae(self):
+        return sum(self.abs_error)
+    
+    @property
+    def mse(self):
+        return sum(self.abs_error**2) / self.__num_data
+    
+    @property
+    def mae(self):
+        return sum(self.abs_error) / self.__num_data
+    
+    @property
+    def rmse(self):
+        return (sum(self.abs_error**2) / self.__num_data)**0.5
+    
+    @property
+    def mape(self):
+        return sum(self.rel_error) / self.__num_data
 
 
 
